package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesPartService;
import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesShiftService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesConfigService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionRecordService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesWorkOrderExtService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesWorkOrderService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.util.MesPcnException;
import cn.estsh.i3plus.mes.pcn.api.iservice.busi.ISyncFuncService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.MathOperation;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.repository.shipping.MesWorkOrderToWmsRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.date.format.DateParser;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Description :
 * @Reference :
 * @Author :
 * @CreateDate : 2024-05-09 10:54
 * @Modify:
 **/

@Service
@Slf4j
public class MesWorkOrderService implements IMesWorkOrderService {

    public static final Logger LOGGER = LoggerFactory.getLogger(MesWorkOrderService.class);
    @Autowired
    private MesWorkOrderRepository workOrderRepository;

    @Autowired
    private IMesWorkOrderExtService workOrderExtService;

    @Autowired
    private MesProductVersionRepository mesProductVersionRDao;
    @Autowired
    private MesBomRepository mesBomRDao;
    @Autowired
    private MesWorkCenterRepository mesWorkCenterRDao;
    @Autowired
    private MesPartRepository mesPartRDao;
    @Autowired
    private MesProductPlanRepository mesProductPlanRDao;
    @Autowired
    private MesProductOffLineRepository mesProductOffLineRDao;

    @Autowired
    private MesProductionRecordRepository productionRecordRao;
    @Autowired
    private MesWorkOrderRepository mesWorkOrderRDao;
    @Autowired
    private MesProduceSnRepository mesProduceSnRao;
    @Autowired
    private ISyncFuncService syncFuncService;

    @Autowired
    private MesWorkOrderLogRepository mesWorkOrderLogRepository;
    @Autowired
    private MesWorkOrderToWmsRepository mesWorkOrderToWmsRDao;
    @Autowired
    private MesShiftRepository mesShiftRepository;
    @Autowired
    private IMesConfigService configService;

    @Autowired
    private IMesPartService iMesPartService;

    @Autowired
    private IMesShiftService mesShiftService;

    @Autowired
    private MesPartSapRepository partSapRepository;

    @Autowired
    private IMesProductionRecordService productionRecordService;

    @Override
    public MesWorkOrder queryMesWorkOrder(MesWorkOrder workOrder) {

        DdlPackBean packBean = getDdlPackBean(workOrder);
        MesWorkOrder result = workOrderRepository.getByProperty(packBean);

        if (StringUtils.isEmpty(result)) {
            throw new ImppBusiException("查询工单不存在，请检查数据");
        }

        return result;
    }

    @Override
    public List<MesWorkOrder> queryMesWorkOrderList(MesWorkOrder workOrder) {

        DdlPackBean packBean = getDdlPackBean(workOrder);

        return workOrderRepository.findByHqlWhere(packBean);
    }

    @Override
    public ListPager<MesWorkOrder> queryMesWorkOrderListByPager(MesWorkOrder workOrder, Pager pager) {


        DdlPackBean packBean = getDdlPackBean(workOrder);

        pager = PagerHelper.getPager(pager, workOrderRepository.findByHqlWhereCount(packBean));

        List<MesWorkOrder> resultList = workOrderRepository.findByHqlWherePage(packBean, pager);

        return new ListPager<>(resultList, pager);
    }

    private DdlPackBean getDdlPackBean(MesWorkOrder workOrder) {

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(workOrder.getOrganizeCode());
        DdlPreparedPack.getStringLikerPack(workOrder.getWorkCenterCode(), "workCenterCode", packBean);
        DdlPreparedPack.getStringLikerPack(workOrder.getWorkCellCode(), "workCellCode", packBean);
        DdlPreparedPack.getStringLikerPack(workOrder.getWorkOrderNo(), "workOrderNo", packBean);

        if (!StringUtils.isEmpty(workOrder.getWorkOrderStatus())) DdlPreparedPack.getNumEqualPack(workOrder.getStatusList(), "workOrderStatus", packBean);
        else DdlPreparedPack.getInPackList(Stream.of(MesExtEnumUtil.ORDER_STATUS.RELEASE.getValue(), MesExtEnumUtil.ORDER_STATUS.PROCESS.getValue()).collect(Collectors.toList()), "workOrderStatus", packBean);

        DdlPreparedPack.getStringBiggerPack(workOrder.getStartTimeStart(), "planStartTime", packBean);
        DdlPreparedPack.getStringSmallerPack(workOrder.getStartTimeEnd(), "planStartTime", packBean);
        DdlPreparedPack.getStringBiggerPack(workOrder.getEndTimeStart(), "planEndTime", packBean);
        DdlPreparedPack.getStringSmallerPack(workOrder.getEndTimeEnd(), "planEndTime", packBean);

        return packBean;
    }


    @Override
    public void doProductReport(List<MesProduceSn> mesProduceSnList, String organizeCode, String userName) {
        //新增初始化 todo
        Map<String, List<MesProduceSn>> mesWorkOrderMap = mesProduceSnList.stream().collect(Collectors.groupingBy(MesProduceSn::getPartNo));
        Map<String,MesPart> mesPartMap = new HashMap<>();
        MesPart mesPart;
        //查询工单状态
        Integer[] orderStatus =new Integer[]{MesExtEnumUtil.ORDER_STATUS.RELEASE.getValue(),MesExtEnumUtil.ORDER_STATUS.PROCESS.getValue()};
        for (Map.Entry<String, List<MesProduceSn>> mesProduceSn : mesWorkOrderMap.entrySet()) {
            List<MesProduceSn> mesWorkOrderList = mesProduceSn.getValue();
            //获取物料信息
            if(mesPartMap.containsKey(mesProduceSn.getKey())){
                mesPart = mesPartMap.get(mesProduceSn.getKey());
            }else{
                //查询物料信息
                DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
                DdlPreparedPack.getStringEqualPack(mesProduceSn.getKey(), "partNo", ddlPackBean);
                mesPart = mesPartRDao.getByProperty(ddlPackBean);
                if(Objects.isNull(mesPart)){
                    throw new ImppBusiException(String.format("物料【%s】信息不存在", mesProduceSn.getKey()));
                }
                mesPartMap.put(mesProduceSn.getKey(),mesPart);
            }
            //根据物料获取已发布的工单
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(mesPart.getPartNo(), "partNo", ddlPackBean);
            DdlPreparedPack.getInPackArray(orderStatus, "workOrderStatus", ddlPackBean);
            MesWorkOrder oldMesWorkOrder = workOrderRepository.getByProperty(ddlPackBean);
            if(Objects.isNull(oldMesWorkOrder)){
                throw new ImppBusiException(String.format("物料【%s】状态未已发布的工单信息不存在", mesPart.getPartNo()));
            }
            //查询工作中心
            ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(oldMesWorkOrder.getWorkCenterCode(), "workCenterCode", ddlPackBean);
            MesWorkCenter mesWorkCenter = mesWorkCenterRDao.getByProperty(ddlPackBean);
            if (Objects.isNull(mesWorkCenter)) {
                throw new ImppBusiException(String.format("产线【%s】不存在", oldMesWorkOrder.getWorkCenterCode()));
            }
            //获取生产版本
            ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(mesPart.getPartNo(), "partNo", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(mesWorkCenter.getErpWorkCenter(), "workCenterCode", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(oldMesWorkOrder.getProductVersion(), "productVersion", ddlPackBean);
            MesProductVersion mesProductVersion = mesProductVersionRDao.getByProperty(ddlPackBean);
            if (null == mesProductVersion) {
                throw new ImppBusiException(String.format("物料【%s】生产版本信息不存在", mesPart.getPartNo()));
            }
            //物料+生产版本获取bom信息
            ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(mesPart.getPartNo(), "partNo", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(mesProductVersion.getAlternativePartList(), "bomVersion", ddlPackBean);
            List<MesBom> mesBoms = mesBomRDao.findByHqlWhere(ddlPackBean);
            if (CollectionUtils.isEmpty(mesBoms)) {
                throw new ImppBusiException(String.format("物料【%s】生产版本【%s】对应bom信息不存在",
                        mesPart.getPartNo(), mesProductVersion.getProductVersion()));
            }
            oldMesWorkOrder.setNum(mesWorkOrderList.size());
            oldMesWorkOrder.setReportedQty(MathOperation.add(oldMesWorkOrder.getNum(), oldMesWorkOrder.getReportedQty()));
            //更新SAP计划完成数量
            saveMesProductPlan(oldMesWorkOrder, false, true);

            //更新工单状态
            double unCompleteQty = MathOperation.sub(oldMesWorkOrder.getQty(), oldMesWorkOrder.getReportedQty());
            oldMesWorkOrder.setUnCompleteQty(unCompleteQty > 0 ? unCompleteQty : 0);
            if (oldMesWorkOrder.getReportedQty() > oldMesWorkOrder.getQty()) {
                throw new ImppBusiException(String.format("工单报工数量【%s】大于工单数量【%s】,不允许报工",
                        oldMesWorkOrder.getReportedQty(), oldMesWorkOrder.getQty()));
            } else if (Objects.equals(oldMesWorkOrder.getReportedQty(), oldMesWorkOrder.getQty())) {
                oldMesWorkOrder.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.COMPLETE.getValue());
            } else {
                oldMesWorkOrder.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.PROCESS.getValue());
            }
            ConvertBean.serviceModelUpdate(oldMesWorkOrder,userName);
            workOrderExtService.update(oldMesWorkOrder);
            //保存数据
            List<MesProductOffLine> mesProductOffLineList = new ArrayList<>();
            MesProductOffLine newMesProductOffLine;
            for (MesProduceSn produceSn : mesWorkOrderList) {
                for (MesBom mesBom : mesBoms) {
                    newMesProductOffLine = new MesProductOffLine();
                    newMesProductOffLine.setReportPartNo(oldMesWorkOrder.getPartNo());
                    newMesProductOffLine.setReportPartNameRdd(oldMesWorkOrder.getPartName());
                    newMesProductOffLine.setItemPartNo(mesBom.getItemPartNo());
                    newMesProductOffLine.setItemPartName(mesBom.getItemPartName());
                    newMesProductOffLine.setItemQty(mesBom.getItemQty());
                    newMesProductOffLine.setAlort(mesProductVersion.getReceiveInventoryPoint());
                    newMesProductOffLine.setStgeLoc(mesProductVersion.getShipInventoryPoint());
                    newMesProductOffLine.setQty(1d);
                    newMesProductOffLine.setReportSn(produceSn.getProductSn());
                    newMesProductOffLine.setBomVersion(oldMesWorkOrder.getProductVersion());
                    newMesProductOffLine.setSerialNumber(produceSn.getSerialNumber());
                    newMesProductOffLine.setUnit(mesBom.getUnit());
                    newMesProductOffLine.setItemUnit(mesBom.getItemUnit());
                    newMesProductOffLine.setWorkOrderNo(oldMesWorkOrder.getWorkOrderNo());
                    newMesProductOffLine.setWorkCenterCode(oldMesWorkOrder.getWorkCenterCode());
                    newMesProductOffLine.setWorkCellCode(oldMesWorkOrder.getWorkCellCode());
                    newMesProductOffLine.setReportType(oldMesWorkOrder.getReportType());
                    newMesProductOffLine.setSapWorkCenter(oldMesWorkOrder.getErpWorkCenter());
                    newMesProductOffLine.setOrganizeCode(oldMesWorkOrder.getOrganizeCode());

                    ConvertBean.serviceModelInitialize(newMesProductOffLine, userName);
                    mesProductOffLineList.add(newMesProductOffLine);
                }
            }
            mesProductOffLineRDao.saveAll(mesProductOffLineList);
        }
    }
    public void doPcnJobProductReport(List<MesProductionRecord> mesProductionRecordList,List<MesShift> mesShiftList,String organizeCode, String userName ,MesWorkOrder mesWorkOrderDb) {

        //查询物料信息
        DdlPackBean ddlPackBeanPart = DdlPackBean.getDdlPackBean(mesWorkOrderDb.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesWorkOrderDb.getPartNo(), "partNo", ddlPackBeanPart);
        MesPart mesPart = mesPartRDao.getByProperty(ddlPackBeanPart);
        if (null == mesPart) {
            MesPcnException.throwMesBusiException("物料【%s】信息不存在", mesWorkOrderDb.getPartNo());
        }

        //查询产线信息
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(mesWorkOrderDb.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesWorkOrderDb.getWorkCenterCode(), "workCenterCode", ddlPackBean);
        MesWorkCenter mesWorkCenter = mesWorkCenterRDao.getByProperty(ddlPackBean);
        if (Objects.isNull(mesWorkCenter)) {
            MesPcnException.throwMesBusiException("产线【%s】不存在", mesWorkOrderDb.getWorkCenterCode());
        }
        //获取生产版本
        MesProductVersion mesProductVersion = getMesProductVersion(mesWorkOrderDb);
        //物料+生产版本获取bom信息
        List<MesBom> mesBoms =  findBomList(mesPart.getOrganizeCode(), mesPart, mesProductVersion.getAlternativePartList());

        LOGGER.info("产线:{} 零件:{}更新工单", mesWorkOrderDb.getWorkCenterCode(), mesWorkOrderDb.getPartNo());
        //更新工单
       // updateMesWorkOrder(mesWorkOrderDb, userName );

        //保存数据
//        List<MesProductOffLine> mesProductOffLineList = new ArrayList<>();
//        for (MesBom mesBom : mesBoms) {
//            mesProductOffLineList.add(creatMesProductOffLine(mesWorkOrder, mesProductVersion, sn, mesBom, false));
//        }
//        mesProductOffLineService.insertBatch(mesProductOffLineList);

        LOGGER.info("产线:{} 零件:{}更新工单mesProduceSns:{}", mesWorkOrderDb.getWorkCenterCode(), mesWorkOrderDb.getPartNo(),mesProductionRecordList.size());
        //保存数据
        List<MesProductOffLine> mesProductOffLineList = new ArrayList<>();
        MesProductOffLine newMesProductOffLine;
        for (MesProductionRecord mesProductionRecord : mesProductionRecordList) {
            for (MesBom mesBom : mesBoms) {
                newMesProductOffLine = new MesProductOffLine();
                newMesProductOffLine.setReportPartNo(mesWorkOrderDb.getPartNo());
                newMesProductOffLine.setReportPartNameRdd(mesWorkOrderDb.getPartName());
                newMesProductOffLine.setItemPartNo(mesBom.getItemPartNo());
                newMesProductOffLine.setItemPartName(mesBom.getItemPartName());
                newMesProductOffLine.setItemQty(mesBom.getItemQty());
                newMesProductOffLine.setAlort(mesProductVersion.getReceiveInventoryPoint());
                newMesProductOffLine.setStgeLoc(mesProductVersion.getShipInventoryPoint());
                newMesProductOffLine.setQty(1d);
                newMesProductOffLine.setReportSn(mesProductionRecord.getProductSn());
                newMesProductOffLine.setBomVersion(mesWorkOrderDb.getProductVersion());
                newMesProductOffLine.setSerialNumber(mesProductionRecord.getSerialNumber());
                newMesProductOffLine.setUnit(mesBom.getUnit());
                newMesProductOffLine.setItemUnit(mesBom.getItemUnit());
                newMesProductOffLine.setWorkOrderNo(mesWorkOrderDb.getWorkOrderNo());
                newMesProductOffLine.setWorkCenterCode(mesWorkOrderDb.getWorkCenterCode());
                newMesProductOffLine.setWorkCellCode(mesWorkOrderDb.getWorkCellCode());
                newMesProductOffLine.setReportType(mesWorkOrderDb.getReportType());
                newMesProductOffLine.setSapWorkCenter(mesWorkOrderDb.getErpWorkCenter());
                newMesProductOffLine.setOrganizeCode(organizeCode);

                ConvertBean.serviceModelInitialize(newMesProductOffLine, userName);
                mesProductOffLineList.add(newMesProductOffLine);
            }
        }
        mesProductOffLineRDao.saveAll(mesProductOffLineList);
    }
    private void updateMesWorkOrder(MesWorkOrder mesWorkOrder, String userName ) {
        //修改工单，需要重新同步
        mesWorkOrder.setSystemSyncStatus(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        ConvertBean.serviceModelUpdate(mesWorkOrder, userName);
        workOrderExtService.update(mesWorkOrder);
        //保存记录
        saveMesWorkOrderLog(mesWorkOrder);
    }
    public void saveMesWorkOrderLog(MesWorkOrder mesWorkOrder) {
        MesWorkOrderLog workOrderLog = new MesWorkOrderLog();
        BeanUtils.copyProperties(mesWorkOrder, workOrderLog, "id");
        mesWorkOrderLogRepository.insert(workOrderLog);
    }
    public List<MesBom> findBomList(String organizeCode, MesPart mesPart, String bomVersion) {
        List<MesBom> bomData = new ArrayList<>();
        // 递归展开 BOM
        expendBomChild(organizeCode, mesPart.getPartNo(), bomVersion, bomData);

        if (bomData.size() == 0) {
            MesPcnException.throwMesBusiException("零件号 %s 没找找到 ERP 同步的原始 BOM 数据，请检查接口是否正常！", mesPart.getPartNo());
        }

        Iterator<MesBom> iterator = bomData.iterator();
        while (iterator.hasNext()) {
            MesBom item = iterator.next();
            // 删除虚零件
            if (Objects.equals("X", item.getPartType())) {
                iterator.remove();
            } else {
                item.setPartNo(mesPart.getPartNo());
                item.setPartName(mesPart.getPartName());
            }
        }

        Map<String, MesBom> summaryBomMap = new HashMap<>();

        for (MesBom item : bomData) {
            String key = String.format("%s:%s", item.getPartNo().toUpperCase().trim(),
                    item.getItemPartNo().toUpperCase().trim());
            if (!summaryBomMap.containsKey(key)) {
                summaryBomMap.put(key, item);
            } else {
                MesBom tempItem = summaryBomMap.get(key);
                // 子零件重复的累加数量
                tempItem.setItemQty(MathOperation.add(tempItem.getItemQty(), item.getItemQty()));
            }
        }
        // 返回汇总数据
        return new ArrayList<>(summaryBomMap.values());
    }
    /**
     * 递归展开 BOM
     *
     * @param organizeCode 工厂代码
     * @param partNo       父零件号
     * @param bomData      存放 BOM 数据的集合
     */
    private void expendBomChild(String organizeCode, String partNo, String bomVersion, List<MesBom> bomData) {
        List<MesBom> bomDetails =  findMesBomByPartNoAndBomVersion(partNo, organizeCode, bomVersion);

        for (MesBom item : bomDetails) {
            MesBom newItem = new MesBom();
            BeanUtils.copyProperties(item, newItem);
            // 处理父零件用量不为 1 的情况
            if (newItem.getQty() != 1.0 && newItem.getQty() > 0.0) {
                Double rateQty = MathOperation.div(newItem.getItemQty(), newItem.getQty(), 8);
                newItem.setQty(1.0);
                newItem.setItemQty(rateQty);
            }
            // 加入到 BOM 清单中
            bomData.add(newItem);
            // 如果有子阶，继续递归
            if (Objects.equals("X", item.getPartType())) {
                expendBomChild(organizeCode, newItem.getItemPartNo(), null, bomData);
            }
        }
    }
    public List<MesBom> findMesBomByPartNoAndBomVersion(String partNo, String organizeCode, String bomVersion) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(partNo, "partNo", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(bomVersion, "bomVersion", ddlPackBean);
        DdlPreparedPack.getStringSmallerPack(TimeTool.getNowTime(true), "effStartTime", ddlPackBean);
        DdlPreparedPack.getOrderBy("effStartTime", CommonEnumUtil.ASC_OR_DESC.DESC.getValue(), ddlPackBean);
        List<MesBom> mesBoms = mesBomRDao.findByHqlWhere(ddlPackBean);
        if (org.springframework.util.CollectionUtils.isEmpty(mesBoms)) {
            MesPcnException.throwMesBusiException("物料【%s】生产版本【%s】对应bom信息不存在", partNo, bomVersion);
        }
        MesBom next = mesBoms.iterator().next();
        //如果存在多个bomCode取最新的一个
        return mesBoms.stream().collect(Collectors.groupingBy(MesBom::getBomCode)).get(next.getBomCode());
    }
    private MesProductVersion getMesProductVersion(MesWorkOrder mesWorkOrder) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(mesWorkOrder.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesWorkOrder.getPartNo(), "partNo", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(mesWorkOrder.getErpWorkCenter(), "workCenterCode", ddlPackBean);
//        DdlPreparedPack.getStringEqualPack(mesWorkOrder.getProductVersion(), "productVersion", ddlPackBean);
        MesProductVersion mesProductVersion = mesProductVersionRDao.getByProperty(ddlPackBean);
        if (null == mesProductVersion) {
            MesPcnException.throwMesBusiException("物料【%s】生产版本信息不存在", mesWorkOrder.getPartNo());
        }
        return mesProductVersion;
    }


    private void updateProductionRecord(String organizeCode, String userName, Long id) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getNumEqualPack(id, "id", ddlPackBean);
//        DdlPreparedPack.getStringEqualPack(sn, "custSn", ddlPackBean);
        productionRecordRao.updateByProperties(new String[]{"modifyUser", "modifyDatetime"},
                new Object[]{userName, TimeTool.getNowTime(true)},ddlPackBean);
    }

    @Override
    @Transactional(propagation= Propagation.REQUIRES_NEW,isolation= Isolation.READ_COMMITTED,readOnly = false,rollbackFor = Exception.class)
    public void doProductReportByRecord(MesProductionRecord productionRecord, String organizeCode, String userName) {
        //新增初始化 todo
        // 根据
        //根据物料获取已发布的工单
        try {
            List<MesWorkOrder> oldMesWorkOrderList = getMesWorkOrder(productionRecord, organizeCode);
            if (CollectionUtils.isEmpty(oldMesWorkOrderList)) {
                LOGGER.error("报工条码:{}找不到加工单", productionRecord.getProductSn());
                updateProductionRecord(organizeCode, userName, productionRecord.getId());
                return;
            }

            report(productionRecord, organizeCode, userName, oldMesWorkOrderList);
        }
        catch (Exception e) {
            String msg = e.getMessage();
            if (e instanceof  ImppBusiException) {
                msg = ((ImppBusiException) e).getErrorMsg();
            }
            e.printStackTrace();
            LOGGER.error("条码:{}报工失败", productionRecord.getProductSn(), e);
            productionRecordService.updateProductionRecord(organizeCode, userName, productionRecord.getId(), msg);
        }
    }
    @Override
    @Transactional(propagation= Propagation.REQUIRES_NEW,isolation= Isolation.READ_COMMITTED,readOnly = false,rollbackFor = Exception.class)
    public void doProductHasOrderReportByRecord(MesProductionRecord productionRecord, String organizeCode, String userName) {
        //新增初始化 todo
        // 根据
        //根据物料获取已发布的工单
        try {
            LOGGER.info("此次报工的条码【{}】,工单【{}】", productionRecord.getProductSn(), productionRecord.getWorkOrderNo());
            List<MesWorkOrder> oldMesWorkOrders = getMesWorkOrders(productionRecord, organizeCode);
            if (CollectionUtils.isEmpty(oldMesWorkOrders)) {
                updateProductionRecord(organizeCode, userName, productionRecord.getId());
                return;
            }
            report(productionRecord, organizeCode, userName, oldMesWorkOrders);
        } catch (Exception e) {
            String msg = e.getMessage();
            if (e instanceof  ImppBusiException) {
                msg = ((ImppBusiException) e).getErrorMsg();
            }
            e.printStackTrace();
            LOGGER.error("条码:{}报工失败", productionRecord.getProductSn(), e);
            productionRecordService.updateProductionRecord(organizeCode, userName, productionRecord.getId(), msg);
        }

    }

    /**
     * 获取最优的工单
     * @param oldMesWorkOrderList
     * @param mesWorkCenter
     * @return
     */
    public MesWorkOrder getBestWorkOrder(MesProductionRecord productionRecord, List<MesWorkOrder> oldMesWorkOrderList, MesWorkCenter mesWorkCenter) {

        if (!StringUtils.isEmpty(productionRecord.getWorkOrderNo())) {
            return oldMesWorkOrderList.get(0);
        }

        for (MesWorkOrder mesWorkOrder : oldMesWorkOrderList) {
            double reportdQty = MathOperation.add(1, mesWorkOrder.getReportedQty());
            // 如果是允许，则需要判断是否超过比例
            if (reportdQty  <= mesWorkOrder.getQty()) {
                return mesWorkOrder;
            }
            if (Objects.equals(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue(),mesWorkCenter.getIsCheckOrderQty())) {
                double remainQty = reportdQty - mesWorkOrder.getQty();
                double rate = remainQty / mesWorkOrder.getQty();
                if (rate > mesWorkCenter.getOrderRate()/100) {
                    continue;
                } else {
                    return mesWorkOrder;
                }
            }

        }
        return  null;
    }

    private void report(MesProductionRecord productionRecord, String organizeCode, String userName,  List<MesWorkOrder> oldMesWorkOrderList) {
        //

        DdlPackBean ddlPackBean;
        //查询工作中心
        ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(productionRecord.getWorkCenterCode(), "workCenterCode", ddlPackBean);
        MesWorkCenter mesWorkCenter = mesWorkCenterRDao.getByProperty(ddlPackBean);
        if (Objects.isNull(mesWorkCenter)) {
            LOGGER.error(String.format("产线【%s】不存在", productionRecord.getWorkCenterCode()));
            throw new ImppBusiException(String.format("产线【%s】不存在", productionRecord.getWorkCenterCode()));
        }
        MesWorkOrder oldMesWorkOrder = getBestWorkOrder(productionRecord, oldMesWorkOrderList, mesWorkCenter);
        if (oldMesWorkOrder == null) {
            LOGGER.error(String.format("未找到匹配的加工单, 条码=%s",productionRecord.getCustSn()));
            return;
            //throw new ImppBusiException(String.format("未找到匹配的加工单"));
        }
        //获取生产版本
        ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(productionRecord.getPartNo(), "partNo", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(mesWorkCenter.getErpWorkCenter(), "workCenterCode", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(oldMesWorkOrder.getProductVersion(), "productVersion", ddlPackBean);
        MesProductVersion mesProductVersion = mesProductVersionRDao.getByProperty(ddlPackBean);
        if (null == mesProductVersion) {
            LOGGER.error(String.format("物料【%s】生产版本信息不存在", productionRecord.getPartNo()));
            throw new ImppBusiException(String.format("物料【%s】生产版本信息不存在", productionRecord.getPartNo()));
        }

        //查询物料信息
        DdlPackBean ddlPackBeanPart = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(productionRecord.getPartNo(), "partNo", ddlPackBeanPart);
        MesPart mesPart = mesPartRDao.getByProperty(ddlPackBeanPart);
        if (null == mesPart) {
            MesPcnException.throwMesBusiException("物料【%s】信息不存在", productionRecord.getPartNo());
        }

        List<MesBom> mesBoms = findBomList(organizeCode, mesPart, mesProductVersion.getAlternativePartList());


        if (CollectionUtils.isEmpty(mesBoms)) {
            LOGGER.error(String.format("物料【%s】生产版本【%s】对应bom信息不存在",
                    productionRecord.getPartNo(), mesProductVersion.getProductVersion()));
            throw new ImppBusiException(String.format("物料【%s】生产版本【%s】对应bom信息不存在",
                    productionRecord.getPartNo(), mesProductVersion.getProductVersion()));
        }
        oldMesWorkOrder.setNum(oldMesWorkOrder.getNum() == null ? 1 : oldMesWorkOrder.getNum()  + 1);
        LOGGER.info("工单报工原数量【{}】",  oldMesWorkOrder.getReportedQty());
        oldMesWorkOrder.setReportedQty(MathOperation.add(oldMesWorkOrder.getNum(), oldMesWorkOrder.getReportedQty()));
        if (StringUtils.isEmpty(productionRecord.getWorkOrderNo())) {
            oldMesWorkOrder.setCompleteQty(oldMesWorkOrder.getReportedQty());
        }
        //更新SAP计划完成数量
        saveMesProductPlan(oldMesWorkOrder, false, true);

        //更新工单状态
        double unCompleteQty = MathOperation.sub(oldMesWorkOrder.getQty(), oldMesWorkOrder.getReportedQty());
        oldMesWorkOrder.setUnCompleteQty(unCompleteQty > 0 ? unCompleteQty : 0);
        if (oldMesWorkOrder.getReportedQty() >= oldMesWorkOrder.getQty()) {
            oldMesWorkOrder.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.COMPLETE.getValue());
        }  else {
            oldMesWorkOrder.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.PROCESS.getValue());
        }
        LOGGER.info("工单报工后数量【{}】",  oldMesWorkOrder.getReportedQty());
        ConvertBean.serviceModelUpdate(oldMesWorkOrder, userName);
        workOrderExtService.update(oldMesWorkOrder);

        //保存数据
        List<MesProductOffLine> mesProductOffLineList = new ArrayList<>();
        MesProductOffLine newMesProductOffLine;
        // 排除重复的数据
        Map<MesBom, List<MesBom>> map = mesBoms.stream().collect(Collectors.groupingBy(mesBom -> {
            MesBom bom = new MesBom();
            bom.setPartNo(mesBom.getPartNo());
            bom.setItemPartNo(mesBom.getItemPartNo());
            return  bom;
        }));
        for (Map.Entry<MesBom, List<MesBom>> entry : map.entrySet()) {
            MesBom mesBom =  entry.getValue().get(0);
            newMesProductOffLine = new MesProductOffLine();
            newMesProductOffLine.setReportPartNo(oldMesWorkOrder.getPartNo());
            newMesProductOffLine.setReportPartNameRdd(oldMesWorkOrder.getPartName());
            newMesProductOffLine.setItemPartNo(mesBom.getItemPartNo());
            newMesProductOffLine.setItemPartName(mesBom.getItemPartName());
            newMesProductOffLine.setItemQty(mesBom.getItemQty());
            newMesProductOffLine.setAlort(mesProductVersion.getReceiveInventoryPoint());
            newMesProductOffLine.setStgeLoc(mesProductVersion.getShipInventoryPoint());
            newMesProductOffLine.setQty(1d);
            newMesProductOffLine.setReportSn(productionRecord.getProductSn());
            newMesProductOffLine.setBomVersion(oldMesWorkOrder.getProductVersion());
            newMesProductOffLine.setSerialNumber(productionRecord.getSerialNumber());
            newMesProductOffLine.setUnit(mesBom.getUnit());
            newMesProductOffLine.setItemUnit(mesBom.getItemUnit());
            newMesProductOffLine.setWorkOrderNo(oldMesWorkOrder.getWorkOrderNo());
            newMesProductOffLine.setWorkCenterCode(oldMesWorkOrder.getWorkCenterCode());
            newMesProductOffLine.setWorkCellCode(oldMesWorkOrder.getWorkCellCode());
            newMesProductOffLine.setReportType(oldMesWorkOrder.getReportType());
            newMesProductOffLine.setSapWorkCenter(oldMesWorkOrder.getErpWorkCenter());
            newMesProductOffLine.setOrganizeCode(oldMesWorkOrder.getOrganizeCode());

            ConvertBean.serviceModelInitialize(newMesProductOffLine, userName);
            mesProductOffLineList.add(newMesProductOffLine);
        }
        mesProductOffLineRDao.saveAll(mesProductOffLineList);

        // 同步工单信息给加工记录、条码信息
        DdlPackBean snDdlPackBean = DdlPackBean.getDdlPackBean();
        DdlPreparedPack.getNumEqualPack(productionRecord.getId(), "id", snDdlPackBean);
        DdlPreparedPack.getStringEqualPack(organizeCode, "organizeCode", snDdlPackBean);

        mesProduceSnRao.updateByProperties(new String[]{"modifyUser", "modifyDatetime","workOrderNo"},
                new Object[]{userName, TimeTool.getNowTime(true), oldMesWorkOrder.getWorkOrderNo()},snDdlPackBean);

        DdlPackBean packBean = DdlPackBean.getDdlPackBean();
        DdlPreparedPack.getNumEqualPack(productionRecord.getId(), "id", packBean);
        productionRecordRao.updateByProperties(new String[]{"modifyUser", "modifyDatetime","workOrderNo", "reportStatus"},
                new Object[]{userName, TimeTool.getNowTime(true), oldMesWorkOrder.getWorkOrderNo(), MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_20.getValue()},packBean);

        LOGGER.info("条码【{}】报工成功,报工工单【{}】",productionRecord.getProductSn(), oldMesWorkOrder.getWorkOrderNo());

    }

    /**
     * 获取对应的工单
     * @param productionRecord
     * @param organizeCode
     * @return
     */
    private List<MesWorkOrder> getMesWorkOrder(MesProductionRecord productionRecord, String organizeCode) {
        MesWorkOrder mesWorkOrder = new MesWorkOrder();

        List<MesWorkOrder> workOrders = new ArrayList<>();
        /**
         * 根据产线物料获取所有工单
         */
        List<MesWorkOrder> oldMesWorkOrders = getMesWorkOrders(productionRecord, organizeCode);
        if (CollectionUtils.isEmpty(oldMesWorkOrders)) {
            return null;
        }
        // 获取当前班次
        String shiftCode = productionRecord.getShiftCode();
        String workCenterCode = productionRecord.getWorkCenterCode();

        //查询工作中心
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(workCenterCode, "workCenterCode", ddlPackBean);
        MesWorkCenter mesWorkCenter = mesWorkCenterRDao.getByProperty(ddlPackBean);
        if (Objects.isNull(mesWorkCenter)) {
            MesPcnException.throwMesBusiException("产线【%s】不存在",workCenterCode);
        }

        List<MesShift> shiftList = mesShiftService.queryMesShift(organizeCode, workCenterCode);
        // 如果当前班次是早班
        if (CollectionUtils.isEmpty(shiftList)) {
            MesPcnException.throwFlowException(String.format("班次信息未维护,产线=%s"), workCenterCode);
        }
        List<MesShift> shifts = shiftList.stream().filter(mesShift ->  Objects.equals(mesShift.getWorkCenterCode(), workCenterCode)).collect(Collectors.toList());

        MesShift currentShift = shifts.stream().filter(mesShift -> Objects.equals(mesShift.getShiftCode(), shiftCode)).findFirst().orElse(null);


        String orderDate = productionRecord.getLotNo();
        if (currentShift.getStartTime().compareTo(currentShift.getEndTime()) > 0){
            // 如果比0点大，需要找日期前一天的班次的工单号,如果是0点前，date =lotNO  如果是0点后，则找lot的前一天
            int hours = transfer(productionRecord.getCompleteDateTime()).getHours();
            if (hours < 12) {
                orderDate = getDate(productionRecord.getLotNo(), 1);
            } else {
                orderDate = getDate(productionRecord.getLotNo(), 0);
            }
        }

        List<MesWorkOrder> list = new ArrayList<>();
        for (MesWorkOrder oldMesWorkOrder : oldMesWorkOrders) {
            if (Objects.equals(oldMesWorkOrder.getShiftCode(), productionRecord.getShiftCode()) && Objects.equals(oldMesWorkOrder.getProduceTime(), orderDate)) {
                list.add(oldMesWorkOrder);
            }
        }
        // 如果找不到则再根据计划开始时间找
        if (CollectionUtils.isEmpty(list)) {
            String beginTime =productionRecord.getLotNo() + " " + currentShift.getStartTime();
            Date beginDate = transfer(beginTime);
            list = oldMesWorkOrders.stream().filter(workOrder ->!StringUtils.isEmpty(workOrder.getPlanStartTime()) && !StringUtils.isEmpty(workOrder.getPlanEndTime()) &&
                    beginDate.getTime() >= transfer(workOrder.getPlanStartTime()).getTime()   &&  beginDate.getTime() <= transfer(workOrder.getPlanEndTime()).getTime()).collect(Collectors.toList());

        }
        workOrders = list;


        MesWorkOrder bestWorkOrder = getBestWorkOrder(productionRecord, workOrders, mesWorkCenter);

        String date = productionRecord.getLotNo();
        // 如果找不到需要找 当天的其它班次
        if (CollectionUtils.isEmpty(workOrders) || bestWorkOrder == null) {
            // 如果开始时间大于结束时间，说明跨天了，跨天说明是晚班,需要找, 如果是晚班, 则找早中班的工单
            if (currentShift.getStartTime().compareTo(currentShift.getEndTime()) > 0) {
                // 如果比0点大，需要找日期前一天的班次的工单号,如果是0点前，date =lotNO  如果是0点后，则找lot的前一天
                int hours = transfer(productionRecord.getCompleteDateTime()).getHours();
                if (hours < 12) {
                    date = getDate(productionRecord.getLotNo(), 1);
                } else {
                    date = getDate(productionRecord.getLotNo(), 0);
                }
                String endTime = date + " " +currentShift.getStartTime();
                String startTime = date + " " +currentShift.getEndTime();

                workOrders = oldMesWorkOrders.stream().filter(workOrder -> !StringUtils.isEmpty(workOrder.getPlanStartTime()) && !StringUtils.isEmpty(workOrder.getPlanEndTime()) &&
                        transfer(startTime).getTime() > transfer(workOrder.getPlanStartTime()).getTime() && transfer(endTime).getTime() < transfer(workOrder.getPlanEndTime()).getTime()).collect(Collectors.toList());

            } else {
                // 早班或者中班 则找当天非当前班次的最早单号
                String startTime = date + " " +currentShift.getEndTime();
                String endTime = getDate(date, 2) + " " +currentShift.getStartTime();
                for (MesWorkOrder workOrder : oldMesWorkOrders) {
                    if (transfer(startTime).getTime() >= transfer(workOrder.getPlanStartTime()).getTime() && transfer(endTime).getTime() <= transfer(workOrder.getPlanEndTime()).getTime()) {
                        break;
                    }
                }
                workOrders = oldMesWorkOrders.stream().filter(workOrder -> transfer(startTime).getTime() >= transfer(workOrder.getPlanStartTime()).getTime() && transfer(endTime).getTime() <= transfer(workOrder.getPlanEndTime()).getTime()).collect(Collectors.toList());
            }
            if (CollectionUtils.isEmpty(workOrders)) {
                // todo 需要记录并且第二天要用job跑
                return null;
            }
        }
        return workOrders;
    }

    private List<MesWorkOrder> getMesWorkOrders(MesProductionRecord productionRecord, String organizeCode) {
        List<Integer> list = Stream.of(MesExtEnumUtil.ORDER_STATUS.RELEASE.getValue(), MesExtEnumUtil.ORDER_STATUS.PROCESS.getValue(), MesExtEnumUtil.ORDER_STATUS.COMPLETE.getValue()).collect(Collectors.toList());

        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(productionRecord.getPartNo(), "partNo", ddlPackBean);

        DdlPreparedPack.getStringEqualPack(productionRecord.getWorkCenterCode(), "workCenterCode", ddlPackBean);
        if (!StringUtils.isEmpty(productionRecord.getWorkOrderNo())) {
            DdlPreparedPack.getStringEqualPack(productionRecord.getWorkOrderNo(), "workOrderNo", ddlPackBean);
        }
        DdlPreparedPack.getInPackList(list, "workOrderStatus", ddlPackBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.ASC.getValue()}, new String[]{"modifyDatetime"}, ddlPackBean);
        // 新增班次逻辑
        //DdlPreparedPack.
        List<MesWorkOrder> oldMesWorkOrders = workOrderRepository.findByHqlWhere(ddlPackBean);
        if (!CollectionUtils.isEmpty(oldMesWorkOrders)) {
            oldMesWorkOrders = oldMesWorkOrders.stream().filter(mesWorkOrder -> !Objects.equals(MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getValue(), mesWorkOrder.getOrderFlag())).collect(Collectors.toList());
        }
        return oldMesWorkOrders;
    }

    /**
     * flag = 1 减少 1天， = 2 加一天
     * @param lotNo
     * @param flag
     * @return
     */
    public String  getDate(String lotNo, int flag) {
        SimpleDateFormat df=new SimpleDateFormat("yyyy-MM-dd");
        Date d = null;
        try {
            d = df.parse(lotNo);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        Calendar cal=Calendar.getInstance();
        cal.setTime(d);
        if (flag == 1) {
            cal.add(Calendar.DATE, -1); //减1天
        } else if (flag == 2) {
            cal.add(Calendar.DATE, 1); //加1天
        }
        return  df.format(cal.getTime()).substring(0, 10);
    }

    private MesProductVersion getMesProductVersion(MesWorkOrder item, MesWorkCenter mesWorkCenter) {
        DdlPackBean ddlPackBean;
        ddlPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPartNo(), "partNo", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(mesWorkCenter.getErpWorkCenter(), "workCenterCode", ddlPackBean);
        ddlPackBean.setOrderByStr(" order by createDatetime desc ");
        MesProductVersion mesProductVersion = mesProductVersionRDao.getByProperty(ddlPackBean);
        if (Objects.isNull(mesProductVersion)) {
            MesPcnException.throwMesBusiException("物料【%s】工作中心【%s】对应的生产版本信息不存在", item.getPartNo(),
                    mesWorkCenter.getErpWorkCenter());
        }
        return mesProductVersion;
    }
    @Override
    public List<MesProductionRecord> getUnReportMesProduceSn(String organizeCode) {

        //1.获取生产加工记录production_record表中，report_status为待汇报状态的工单
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        List<Integer> statusList = Stream.of(MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_10.getValue(), MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_40.getValue()).collect(Collectors.toList());
        DdlPreparedPack.getInPackList( statusList,"reportStatus",ddlPackBean);
        DdlPreparedPack.getIsNull("workOrderNo",ddlPackBean);
        String workCenterCodes = configService.getCfgValue(organizeCode, "MES_PCN_REPORT_CENTER");

        if (!StringUtils.isEmpty(workCenterCodes)) {
            List<String> centerList = Arrays.asList(workCenterCodes.split(","));
            DdlPreparedPack.getNotInPack(centerList, "workCenterCode", ddlPackBean);
        }
        /*String workCenterCodes ="K03CK01A";
        if (!StringUtils.isEmpty(workCenterCodes)) {
            List<String> centerList = Arrays.asList(workCenterCodes.split(","));
            DdlPreparedPack.getInPack(centerList, "workCenterCode", ddlPackBean);
        }*/
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.ASC.getValue()}, new String[]{"modifyDatetime"}, ddlPackBean);

        String numStr = configService.getCfgValue(organizeCode, "MES_PCN_REPORT_NUM");
        int num = StringUtils.isEmpty(numStr) ? 50 : Integer.valueOf(numStr);

        List<MesProductionRecord> mesProductionRecordList = productionRecordRao.findByHqlTopWhere(ddlPackBean, num);
        List<MesProductionRecord> recordList = new ArrayList<>();
        // 根据零件号分组
        /*if(!CollectionUtils.isEmpty(mesProductionRecordList)) {
            Map<String, List<MesProductionRecord>> map = mesProductionRecordList.stream().collect(Collectors.groupingBy(MesProductionRecord::getPartNo));
            map.forEach((k, v) -> {
                recordList.add(v.get(0));
            });
        }*/
        recordList.addAll(mesProductionRecordList);

        return recordList;
    }

    @Override
    public List<MesProductionRecord> getUnReportHasOrderMesProduceSn(String organizeCode) {
        //1.获取生产加工记录production_record表中，report_status为待汇报状态的工单
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        List<Integer> statusList = Stream.of(MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_10.getValue(), MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_40.getValue()).collect(Collectors.toList());
        DdlPreparedPack.getInPackList( statusList,"reportStatus",ddlPackBean);
        DdlPreparedPack.getIsNotNull("workOrderNo",ddlPackBean);
        String workCenterCodes = configService.getCfgValue(organizeCode, "MES_PCN_REPORT_CENTER");
        //String workCenterCodes ="B19CK01A";
        if (!StringUtils.isEmpty(workCenterCodes)) {
            List<String> centerList = Arrays.asList(workCenterCodes.split(","));
            DdlPreparedPack.getNotInPack(centerList, "workCenterCode", ddlPackBean);
        }
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.ASC.getValue()}, new String[]{"modifyDatetime"}, ddlPackBean);
        List<MesProductionRecord> mesProductionRecordList = productionRecordRao.findByHqlTopWhere(ddlPackBean, 50);
        List<MesProductionRecord> recordList = new ArrayList<>();
        // 根据零件号分组
        /*if(!CollectionUtils.isEmpty(mesProductionRecordList)) {
            Map<String, List<MesProductionRecord>> map = mesProductionRecordList.stream().collect(Collectors.groupingBy(MesProductionRecord::getPartNo));
            map.forEach((k, v) -> {
                DdlPackBean packBean = DdlPackBean.getDdlPackBean();
                DdlPreparedPack.getStringEqualPack(k, "partNo", packBean);
                // 排除掉试制造件
                MesPartSap part = partSapRepository.getByProperty(packBean);
                if (part != null && !Objects.equals(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue(), part.getIsTestPiece())) {
                    recordList.add(v.get(0));
                }
            });
        }*/
        recordList.addAll(mesProductionRecordList);

        return recordList;
    }


    @Override
    public List<MesProductionRecord> getPreDayReportMesProductionRecord(String organizeCode,List<MesShift> mesShiftList, String  workCenterCode ) {

        //1.获取生产加工记录production_record表中，report_status为待汇报状态的工单
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(workCenterCode,"workCenterCode",ddlPackBean);
//        DdlPreparedPack.getNumEqualPack( MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_10.getValue(),"reportStatus",ddlPackBean);
        DdlPreparedPack.getInPackList(
                Arrays.asList(MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_10.getValue(),
                MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_40.getValue()) ,"reportStatus",ddlPackBean);
        List<MesShift> collect = mesShiftList.stream().filter(s -> s.getShiftCode().equalsIgnoreCase("10")).collect(Collectors.toList());
        String startTime=" 08:00:00";
        if(!CollectionUtils.isEmpty(collect)){
            MesShift mesShift= collect.get(0);
            startTime= StringUtil.isEmpty(mesShift.getStartTime())?" 08:00:00":" "+mesShift.getStartTime();
        }
        //获取前一天的早班到当天早班的区间的所有加工记录，统计个数
        DdlPreparedPack.timeBuilder(
                new SimpleDateFormat("yyyy-MM-dd").format(TimeTool.getDateBefore(new Date(),1)) + startTime,
                TimeTool.getToday() + startTime,
                "completeDateTime", ddlPackBean, false);

        return  productionRecordRao.findByHqlWhere(ddlPackBean);
    }
    @Override
    public List<MesProduceSn> getPreDayReportMesProduceSn(String organizeCode,List<MesProductionRecord> mesProductionRecordList ) {
        DdlPackBean snPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(mesProductionRecordList.stream().map(MesProductionRecord::getProductSn).collect(Collectors.toList()),
                "productSn",snPackBean);
        DdlPreparedPack.getIsNotNull("workCenterCode", snPackBean);
        return mesProduceSnRao.findByHqlWhere(snPackBean);
    }

    private void saveMesProductPlan(MesWorkOrder bean, boolean isInsert, boolean isReport) {
        if (StringUtil.isEmpty(bean.getPlanOrderNo())) {
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(bean.getPartNo(), "planPartNo", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(bean.getPlanStartTime(), "planStartDate", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(bean.getPlanEndTime(), "planEndDate", ddlPackBean);
            MesProductPlan mesProductPlan = mesProductPlanRDao.getByProperty(ddlPackBean);
            if (null == mesProductPlan) {
                MesProductPlan mesPlanOrder = new MesProductPlan();
                mesPlanOrder.setPlanOrderNo("");
                mesPlanOrder.setPlanQty(bean.getQty());
                mesPlanOrder.setPlanPartNo(bean.getPartNo());
                mesPlanOrder.setCompleteQty(0d);
                mesPlanOrder.setUncompleteQty(bean.getQty());
                mesPlanOrder.setUnit(bean.getUnit());
                mesPlanOrder.setPlanStartDate(bean.getPlanStartTime());
                mesPlanOrder.setPlanEndDate(bean.getPlanEndTime());
                mesPlanOrder.setPartMappingWorkCenterCode(bean.getErpWorkCenter());
                mesPlanOrder.setPlanOrganizeCode(bean.getOrganizeCode());
                mesPlanOrder.setOrganizeCode(bean.getOrganizeCode());
                ConvertBean.serviceModelInitialize(mesPlanOrder, bean.getCreateUser());
                mesProductPlanRDao.insert(mesPlanOrder);
            } else {
                if (isInsert) {
                    mesProductPlan.setPlanQty(MathOperation.add(mesProductPlan.getPlanQty(), bean.getQty()));
                } else {
                    //报工
                    if (isReport) {
                        mesProductPlan.setCompleteQty(MathOperation.add(mesProductPlan.getCompleteQty(), bean.getNum()));
                        //报工调整
                    } else {
                        mesProductPlan.setCompleteQty(MathOperation.sub(mesProductPlan.getCompleteQty(), bean.getNum()));
                    }
                }
                mesProductPlan.setUncompleteQty(MathOperation.sub(mesProductPlan.getPlanQty(), mesProductPlan.getCompleteQty()));
                ConvertBean.serviceModelUpdate(mesProductPlan, bean.getCreateUser());
                mesProductPlanRDao.update(mesProductPlan);
            }
        } else {
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(bean.getPlanOrderNo(), "planOrderNo", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(bean.getPartNo(), "planPartNo", ddlPackBean);
            MesProductPlan mesProductPlan = mesProductPlanRDao.getByProperty(ddlPackBean);
            if (null == mesProductPlan) {
                throw new ImppBusiException(String.format("SAP计划单号【%s】物料【%s】,不存在", bean.getPlanOrderNo(), bean.getPartNo()));
            }
            if (!isInsert) {
                if (Objects.isNull(mesProductPlan.getCompleteQty())) {
                    mesProductPlan.setCompleteQty(0d);
                }
                //报工
                if (isReport) {
                    mesProductPlan.setCompleteQty(MathOperation.add(mesProductPlan.getCompleteQty(), bean.getNum()));
                    //报工调整
                } else {
                    mesProductPlan.setCompleteQty(MathOperation.sub(mesProductPlan.getCompleteQty(), bean.getNum()));
                }
                mesProductPlan.setUncompleteQty(MathOperation.sub(mesProductPlan.getPlanQty(), mesProductPlan.getCompleteQty()));
                ConvertBean.serviceModelUpdate(mesProductPlan, bean.getCreateUser());
                mesProductPlanRDao.update(mesProductPlan);
            }
        }
    }

    @Override
    public List<MesWorkOrder> insertMesWorkOrder(List<MesProductionRecord> mesProductionRecordList,List<MesShift> mesShiftList, String organizeCode, String userName) {
        List<MesWorkOrder> mesWorkOrderList=new ArrayList<>();
        Map<String, List<MesProductionRecord>> mesProductionRecordListMap = mesProductionRecordList.stream().collect(Collectors.groupingBy(sn -> sn.getWorkCenterCode() +"="+ sn.getPartNo()));
        for (Map.Entry<String, List<MesProductionRecord>> stringListEntry : mesProductionRecordListMap.entrySet()) {
            List<MesProductionRecord> value = stringListEntry.getValue();
            MesProductionRecord mesProductionRecord = value.get(0);
            String[] split = stringListEntry.getKey().split("=");
            String workCenterCode = split[0];
            String partNo = split[1];
            //查询工作中心
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(workCenterCode, "workCenterCode", ddlPackBean);
            MesWorkCenter mesWorkCenter = mesWorkCenterRDao.getByProperty(ddlPackBean);
            if (Objects.isNull(mesWorkCenter)) {
                MesPcnException.throwMesBusiException("产线【%s】不存在",workCenterCode);
            }
            MesWorkOrder item=new MesWorkOrder();
            item.setOrganizeCode(organizeCode);
            item.setWorkCenterCode(workCenterCode);
            item.setErpWorkCenter(mesWorkCenter.getErpWorkCenter());
            item.setPartNo(partNo);
            item.setWorkOrderType(MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue());
            item.setOrderFlag(MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.N.getValue());
            List<MesShift> collect = mesShiftList.stream().filter(s -> s.getShiftCode().equalsIgnoreCase("20")).collect(Collectors.toList());
            MesShift mesShift= collect.get(0);
            item.setShiftCode(mesShift.getShiftCode());
            item.setShiftName(mesShift.getShiftName());
            String today = TimeTool.getToday();
            String dayBefore = new SimpleDateFormat("yyyy-MM-dd").format(TimeTool.getDateBefore(new Date(), 1));
            item.setPlanStartTime(dayBefore+" "+mesShift.getStartTime());
            item.setPlanEndTime(today+" "+mesShift.getEndTime());

            //生成工单号
            List<String> orderList=new ArrayList<>();
            GenSerialNoModel genSerialNoModel = new GenSerialNoModel("MES_WORK_CENTER_ORDER_NO");
            genSerialNoModel.setPartNo(workCenterCode);
            orderList = syncFuncService.syncSerialNo(genSerialNoModel,userName, organizeCode, 1).getResultList();

            String orderNo = orderList.get(0);
            DdlPackBean orderPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(orderNo, "workOrderNo", orderPackBean);
            if (mesWorkOrderRDao.isExitByHql(orderPackBean)) {
                MesPcnException.throwMesBusiException("单号流水码生成重复");
            }
            item.setWorkOrderNo(orderNo);
            item.setPartName(mesProductionRecord.getPartName());
            //生产时间新增取当天
            item.setProduceTime(TimeTool.getToday());
            item.setUnCompleteQty(0d);
            item.setQty(value.stream().mapToDouble(MesProductionRecord::getQty).sum());
            item.setCompleteQty(item.getQty());
            item.setReportedQty(item.getQty());
            item.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.COMPLETE.getValue());


            //校验标识
            //若工单类型为排序
            List<MesWorkOrderToWms> saveMesWorkOrderToWms = new ArrayList<>();
            if (item.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue()) {
                //校验产线类型和工单类型是否匹配
                if (!Objects.equals(mesWorkCenter.getCenterType(), MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue())) {
                    MesPcnException.throwMesBusiException("产线【%s】类型为【%s】，请选择非排序产线", mesWorkCenter.getWorkCenterCode(),
                            MesExtEnumUtil.WORK_CENTER_TYPE.valueOfDescription(mesWorkCenter.getCenterType()));
                }

                //校验物料生产版本是否存在
                MesProductVersion mesProductVersion = checkMesProductVersion(item);
                item.setProductVersion(mesProductVersion.getProductVersion());
                //获取bom信息
                List<MesBom> bomList =  findMesBomByPartNoAndBomVersion(item.getPartNo(), item.getOrganizeCode(), mesProductVersion.getAlternativePartList());

                //发送工单信息给WMS
                saveMesWorkOrderToWms.addAll(saveMesWorkOrderToWms(item, bomList));
            }

            item.setSystemSyncStatus(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
            item.setFid(UUID.randomUUID().toString());
            ConvertBean.serviceModelInitialize(item, userName);
            MesWorkOrder mesWorkOrder = workOrderRepository.insert(item);
            //保存发送WMS数据
            if (CollectionUtils.isNotEmpty(saveMesWorkOrderToWms)) {
                mesWorkOrderToWmsRDao.saveAll(saveMesWorkOrderToWms);
            }
            saveMesWorkOrderLog(mesWorkOrder);
            mesWorkOrderList.add(mesWorkOrder);
        }

        return mesWorkOrderList;
    }

    public void deleteUnSortOrder(String organizeCode, List<String> partNoList) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(partNoList, "partNo", ddlPackBean);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue(), "workOrderType", ddlPackBean);
        DdlPreparedPack.getStringSmallerNotEqualPack(TimeTool.getToday(), "planStartTime", ddlPackBean);
        DdlPreparedPack.getNumberSmallerPack(MesExtEnumUtil.ORDER_STATUS.CLOSE.getValue(), "workOrderStatus", ddlPackBean);
        workOrderRepository.updateByProperties(new String[]{"modifyUser", "modifyDatetime", "workOrderStatus"},
                new Object[]{"CLOSE_JOB", TimeTool.getNowTime(true), MesExtEnumUtil.ORDER_STATUS.CLOSE.getValue()}, ddlPackBean);
    }
    public List<MesWorkOrderToWms> saveMesWorkOrderToWms(MesWorkOrder mesWorkOrder, List<MesBom> mesBomList) {
        List<MesWorkOrderToWms> mesWorkOrderToWmsList = new ArrayList<>();
        MesWorkOrderToWms mesWorkOrderToWms = null;
        for (MesBom mesBom : mesBomList) {
            mesWorkOrderToWms = new MesWorkOrderToWms();
            BeanUtils.copyProperties(mesWorkOrder, mesWorkOrderToWms, "id");
            mesWorkOrderToWms.setUnit(mesBom.getUnit());
            mesWorkOrderToWms.setItemPartNo(mesBom.getItemPartNo());
            mesWorkOrderToWms.setItemPartName(mesBom.getItemPartName());
            mesWorkOrderToWms.setItemUnit(mesBom.getItemUnit());
            mesWorkOrderToWms.setItemQty(MathOperation.mul(mesWorkOrder.getQty(), mesBom.getItemQty()));
            ConvertBean.serviceModelInitialize(mesWorkOrderToWms, mesWorkOrderToWms.getCreateUser());
            mesWorkOrderToWmsList.add(mesWorkOrderToWms);
        }
        return mesWorkOrderToWmsList;
    }
    private MesProductVersion checkMesProductVersion(MesWorkOrder item) {
        DdlPackBean ddlPackBean;
        ddlPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPartNo(), "partNo", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(item.getErpWorkCenter(), "workCenterCode", ddlPackBean);
        MesProductVersion mesProductVersion = mesProductVersionRDao.getByProperty(ddlPackBean);
        if (Objects.isNull(mesProductVersion)) {
            MesPcnException.throwMesBusiException("物料【%s】产线【%s】生产版本【%s】信息不存在", item.getPartNo(), item.getWorkCenterCode(), item.getProductVersion());
        }
        return mesProductVersion;
    }

    private Date transfer(String dateStr){
        return  DateUtil.parse(dateStr, (DateParser) DatePattern.NORM_DATETIME_FORMAT);
    }
}
